@@ -51,17 +51,28 @@ action inputs, which can't be easily translated into command-line arguments.
     inputs_PYMUPDF_SETUP_MUPDF_BUILD
         Used to directly set PYMUPDF_SETUP_MUPDF_BUILD.
         E.g. 'git:--recursive --depth 1 --shallow-submodules --branch master https://github.com/ArtifexSoftware/mupdf.git'
-    inputs_wheels_implementations
-        Used to directly set PYMUPDF_SETUP_IMPLEMENTATIONS.
-        'a', 'b', 'ab'.
+    inputs_PYMUPDF_SETUP_MUPDF_BUILD_TYPE
+        Used to directly set PYMUPDF_SETUP_MUPDF_BUILD_TYPE. Note that as of
+        2024-09-10 .github/workflows/build_wheels.yml does not set this.
+    Py_LIMITED_API
+        If specified we build a single wheel for all python versions using the Python Limited API.
 
 Building for Pyodide
 
+    If GITHUB_EVENT_NAME is 'schedule' we assume we are running as a scheduled
+    Github workflow, and if inputs_PYMUPDF_SETUP_MUPDF_BUILD is unset we change
+    it to use MuPDF master branch.
+    
     If `inputs_wheels_linux_pyodide` is true and we are on Linux, we clone
     `emsdk.git`, set it up, and run `pyodide build`. This runs our setup.py
     with CC etc set up to create Pyodide binaries in a wheel called, for
     example, `PyMuPDF-1.23.2-cp311-none-emscripten_3_1_32_wasm32.whl`.
 
+Set up for use outside Github
+
+    sudo apt install docker.io
+    sudo usermod -aG docker $USER
+
 Example usage:
 
      PYMUPDF_SETUP_MUPDF_BUILD=../mupdf py -3.9-32 PyMuPDF/scripts/gh_release.py venv build-devel
@@ -71,11 +82,13 @@ import glob
 import inspect
 import os
 import platform
+import re
 import shlex
 import subprocess
 import sys
 import textwrap
 
+import test as test_py
 
 pymupdf_dir = os.path.abspath( f'{__file__}/../..')
 
@@ -101,6 +114,9 @@ def main():
         v = os.environ[ k]
         log( f'    {k}: {v!r}')
     
+    if test_py.github_workflow_unimportant():
+        return
+    
     valgrind = False
     if len( sys.argv) == 1:
         args = iter( ['build'])
@@ -133,6 +149,12 @@ def main():
             pattern = f'{prefix}-*{platform_tag()}.whl'
             paths = glob.glob( pattern)
             log( f'{pattern=} {paths=}')
+            # Follow pipcl.py and look at AUDITWHEEL_PLAT. This allows us to
+            # cope if building for both musl and normal linux.
+            awp = os.environ.get('AUDITWHEEL_PLAT')
+            if awp:
+                paths = [i for i in paths if awp in i]
+                log(f'After selecting AUDITWHEEL_PLAT={awp!r}, {paths=}.')
             paths = ' '.join( paths)
             run( f'pip install {paths}')
         elif arg == 'venv':
@@ -175,12 +197,14 @@ def build( platform_=None, valgrind=False):
     inputs_wheels_linux_aarch64 = get_bool('inputs_wheels_linux_aarch64', inputs_wheels_default)
     inputs_wheels_linux_auto = get_bool('inputs_wheels_linux_auto', inputs_wheels_default)
     inputs_wheels_linux_pyodide = get_bool('inputs_wheels_linux_pyodide', 0)
-    inputs_wheels_macos_arm64 = get_bool('inputs_wheels_macos_arm64', inputs_wheels_default)
+    inputs_wheels_macos_arm64 = get_bool('inputs_wheels_macos_arm64', 0)
     inputs_wheels_macos_auto = get_bool('inputs_wheels_macos_auto', inputs_wheels_default)
     inputs_wheels_windows_auto = get_bool('inputs_wheels_windows_auto', inputs_wheels_default)
     inputs_wheels_cps = os.environ.get('inputs_wheels_cps')
     inputs_PYMUPDF_SETUP_MUPDF_BUILD = os.environ.get('inputs_PYMUPDF_SETUP_MUPDF_BUILD')
-    inputs_wheels_implementations = os.environ.get('inputs_wheels_implementations', 'b')
+    inputs_PYMUPDF_SETUP_MUPDF_BUILD_TYPE = os.environ.get('inputs_PYMUPDF_SETUP_MUPDF_BUILD_TYPE')
+    
+    PYMUPDF_SETUP_PY_LIMITED_API = os.environ.get('PYMUPDF_SETUP_PY_LIMITED_API')
     
     log( f'{inputs_flavours=}')
     log( f'{inputs_sdist=}')
@@ -194,169 +218,276 @@ def build( platform_=None, valgrind=False):
     log( f'{inputs_wheels_windows_auto=}')
     log( f'{inputs_wheels_cps=}')
     log( f'{inputs_PYMUPDF_SETUP_MUPDF_BUILD=}')
+    log( f'{inputs_PYMUPDF_SETUP_MUPDF_BUILD_TYPE=}')
+    log( f'{PYMUPDF_SETUP_PY_LIMITED_API=}')
     
     # Build Pyodide wheel if specified.
     #
     if platform.system() == 'Linux' and inputs_wheels_linux_pyodide:
-        build_pyodide_wheel(inputs_wheels_implementations, inputs_PYMUPDF_SETUP_MUPDF_BUILD)
+        # In scheduled runs (by .github/workflows/test_pyodide.yml), use MuPDF
+        # master.
+        GITHUB_EVENT_NAME = os.getenv('GITHUB_EVENT_NAME')
+        if GITHUB_EVENT_NAME == 'schedule':
+            if inputs_PYMUPDF_SETUP_MUPDF_BUILD in ('', None):
+                log(f'Overriding inputs_PYMUPDF_SETUP_MUPDF_BUILD because {GITHUB_EVENT_NAME=} {inputs_PYMUPDF_SETUP_MUPDF_BUILD=}.')
+                inputs_PYMUPDF_SETUP_MUPDF_BUILD = 'git:--branch master https://github.com/ArtifexSoftware/mupdf.git'
+                log(f'{inputs_PYMUPDF_SETUP_MUPDF_BUILD=}')
+        build_pyodide_wheel(inputs_PYMUPDF_SETUP_MUPDF_BUILD)
     
-    # Build 
+    # Build sdist(s).
     #
-    env_extra = dict()
+    if inputs_sdist:
+        if pymupdf_dir != os.path.abspath( os.getcwd()):
+            log( f'Changing dir to {pymupdf_dir=}')
+            os.chdir( pymupdf_dir)
+        # Create PyMuPDF sdist.
+        run(f'{sys.executable} setup.py sdist')
+        assert glob.glob('dist/PyMuPDF-*.tar.gz')
+        if inputs_flavours:
+            # Create PyMuPDFb sdist.
+            run(
+                    f'{sys.executable} setup.py sdist',
+                    env_extra=dict(PYMUPDF_SETUP_FLAVOUR='b'),
+                    )
+            assert glob.glob('dist/PyMuPDFb-*.tar.gz')
     
-    def set_if_unset(name, value):
-        v = os.environ.get(name)
-        if v is None:
-            log( f'Setting environment {name=} to {value=}')
-            env_extra[ name] = value
-        else:
-            log( f'Not changing {name}={v!r} to {value!r}')
-    set_if_unset( 'CIBW_BUILD_VERBOSITY', '3')
-    set_if_unset( 'CIBW_SKIP', '"pp* *i686 *-musllinux_* cp36* cp37*"')
+    # Build wheels.
+    #
+    if (0
+            or inputs_wheels_linux_aarch64
+            or inputs_wheels_linux_auto
+            or inputs_wheels_macos_arm64
+            or inputs_wheels_macos_auto
+            or inputs_wheels_windows_auto
+            ):
+        env_extra = dict()
     
-    def make_string(*items):
-        ret = list()
-        for item in items:
-            if item:
-                ret.append(item)
-        return ' '.join(ret)
+        def set_if_unset(name, value):
+            v = os.environ.get(name)
+            if v is None:
+                log( f'Setting environment {name=} to {value=}')
+                env_extra[ name] = value
+            else:
+                log( f'Not changing {name}={v!r} to {value!r}')
+        set_if_unset( 'CIBW_BUILD_VERBOSITY', '1')
+        # We exclude pp* because of `fitz_wrap.obj : error LNK2001: unresolved
+        # external symbol PyUnicode_DecodeRawUnicodeEscape`.
+        # 2024-06-05: musllinux on aarch64 fails because libclang cannot find
+        # libclang.so.
+        #
+        # 2024-07-09: exclude Windows 32-bit Python-3.13 because when running
+        # under 32-bit Python-3.13, there is no 64-bit Python-3.13 available
+        # via `py -3.13`, which we need because libclang fails on 32-bit
+        # Windows. Hopefully this will work when Python-3.13 is officially
+        # released.
+        #
+        set_if_unset( 'CIBW_SKIP', 'pp* *i686 cp36* cp37* *musllinux*aarch64* cp313-win32')
     
-    cps = inputs_wheels_cps if inputs_wheels_cps else 'cp38* cp39* cp310* cp311* cp312*'
-    set_if_unset( 'CIBW_BUILD', cps)
+        def make_string(*items):
+            ret = list()
+            for item in items:
+                if item:
+                    ret.append(item)
+            return ' '.join(ret)
     
-    if platform.system() == 'Linux':
-        set_if_unset(
-                'CIBW_ARCHS_LINUX',
-                make_string(
-                    'auto64' * inputs_wheels_linux_auto,
-                    'aarch64' * inputs_wheels_linux_aarch64,
-                    ),
-                )
-        if env_extra.get('CIBW_ARCHS_LINUX') == '':
-            log(f'Not running cibuildwheel because CIBW_ARCHS_LINUX is empty string.')
-            return
+        cps = inputs_wheels_cps if inputs_wheels_cps else 'cp38* cp39* cp310* cp311* cp312*'
+        set_if_unset( 'CIBW_BUILD', cps)
+        for cp in cps.split():
+            m = re.match('cp([0-9]+)[*]', cp)
+            assert m
+            v = int(m.group(1))
+            if v == 313:
+                # Need to set CIBW_PRERELEASE_PYTHONS, otherwise cibuildwheel
+                # will refuse.
+                log(f'Setting CIBW_PRERELEASE_PYTHONS for Python version {cp=}.')
+                set_if_unset( 'CIBW_PRERELEASE_PYTHONS', '1')
     
-    if platform.system() == 'Windows':
-        set_if_unset(
-                'CIBW_ARCHS_WINDOWS',
-                make_string(
-                    'auto' * inputs_wheels_windows_auto,
-                    ),
-                )
-        if env_extra.get('CIBW_ARCHS_WINDOWS') == '':
-            log(f'Not running cibuildwheel because CIBW_ARCHS_WINDOWS is empty string.')
-            return
-    
-    if platform.system() == 'Darwin':
-        set_if_unset(
-                'CIBW_ARCHS_MACOS',
-                make_string(
-                    'auto' * inputs_wheels_macos_auto,
-                    'arm64' * inputs_wheels_macos_arm64,
-                    ),
-                )
-        if env_extra.get('CIBW_ARCHS_MACOS') == '':
-            log(f'Not running cibuildwheel because CIBW_ARCHS_MACOS is empty string.')
-            return
-    
-    def env_pass(name):
-        '''
-        Adds `name` to CIBW_ENVIRONMENT_PASS_LINUX if required to be available
-        when building wheel with cibuildwheel.
-        '''
         if platform.system() == 'Linux':
-            v = env_extra.get('CIBW_ENVIRONMENT_PASS_LINUX', '')
-            if v:
-                v += ' '
-            v += name
-            env_extra['CIBW_ENVIRONMENT_PASS_LINUX'] = v
+            set_if_unset(
+                    'CIBW_ARCHS_LINUX',
+                    make_string(
+                        'auto64' * inputs_wheels_linux_auto,
+                        'aarch64' * inputs_wheels_linux_aarch64,
+                        ),
+                    )
+            if env_extra.get('CIBW_ARCHS_LINUX') == '':
+                log(f'Not running cibuildwheel because CIBW_ARCHS_LINUX is empty string.')
+                return
     
-    def env_set(name, value, pass_=False):
-        assert isinstance( value, str)
-        if not name.startswith('CIBW'):
-            assert pass_, f'{name=} {value=}'
-        env_extra[ name] = value
-        if pass_:
-            env_pass(name)
+        if platform.system() == 'Windows':
+            set_if_unset(
+                    'CIBW_ARCHS_WINDOWS',
+                    make_string(
+                        'auto' * inputs_wheels_windows_auto,
+                        ),
+                    )
+            if env_extra.get('CIBW_ARCHS_WINDOWS') == '':
+                log(f'Not running cibuildwheel because CIBW_ARCHS_WINDOWS is empty string.')
+                return
+    
+        if platform.system() == 'Darwin':
+            set_if_unset(
+                    'CIBW_ARCHS_MACOS',
+                    make_string(
+                        'auto' * inputs_wheels_macos_auto,
+                        'arm64' * inputs_wheels_macos_arm64,
+                        ),
+                    )
+            if env_extra.get('CIBW_ARCHS_MACOS') == '':
+                log(f'Not running cibuildwheel because CIBW_ARCHS_MACOS is empty string.')
+                return
+    
+        def env_pass(name):
+            '''
+            Adds `name` to CIBW_ENVIRONMENT_PASS_LINUX if required to be available
+            when building wheel with cibuildwheel.
+            '''
+            if platform.system() == 'Linux':
+                v = env_extra.get('CIBW_ENVIRONMENT_PASS_LINUX', '')
+                if v:
+                    v += ' '
+                v += name
+                env_extra['CIBW_ENVIRONMENT_PASS_LINUX'] = v
+    
+        def env_set(name, value, pass_=False):
+            assert isinstance( value, str)
+            if not name.startswith('CIBW'):
+                assert pass_, f'Non-CIBW* name requires `pass_` to be true. {name=} {value=}.'
+            env_extra[ name] = value
+            if pass_:
+                env_pass(name)
 
-    if os.environ.get('PYMUPDF_SETUP_LIBCLANG'):
-        env_pass('PYMUPDF_SETUP_LIBCLANG')
+        if os.environ.get('PYMUPDF_SETUP_LIBCLANG'):
+            env_pass('PYMUPDF_SETUP_LIBCLANG')
     
-    env_set('PYMUPDF_SETUP_IMPLEMENTATIONS', inputs_wheels_implementations, pass_=1)
-    if inputs_skeleton:
-        env_set('PYMUPDF_SETUP_SKELETON', inputs_skeleton, pass_=1)
+        if inputs_skeleton:
+            env_set('PYMUPDF_SETUP_SKELETON', inputs_skeleton, pass_=1)
     
-    if inputs_PYMUPDF_SETUP_MUPDF_BUILD not in ('-', None):
-        log(f'Setting PYMUPDF_SETUP_MUPDF_BUILD to {inputs_PYMUPDF_SETUP_MUPDF_BUILD!r}.')
-        env_set('PYMUPDF_SETUP_MUPDF_BUILD', inputs_PYMUPDF_SETUP_MUPDF_BUILD, pass_=True)
-        env_set('PYMUPDF_SETUP_MUPDF_TGZ', '', pass_=True)   # Don't put mupdf in sdist.
+        if inputs_PYMUPDF_SETUP_MUPDF_BUILD not in ('-', None):
+            log(f'Setting PYMUPDF_SETUP_MUPDF_BUILD to {inputs_PYMUPDF_SETUP_MUPDF_BUILD!r}.')
+            env_set('PYMUPDF_SETUP_MUPDF_BUILD', inputs_PYMUPDF_SETUP_MUPDF_BUILD, pass_=True)
+            env_set('PYMUPDF_SETUP_MUPDF_TGZ', '', pass_=True)   # Don't put mupdf in sdist.
     
-    def set_cibuild_test():
-        log( f'set_cibuild_test(): {inputs_skeleton=}')
-        valgrind_text = ''
-        if valgrind:
-            valgrind_text = ' --valgrind 1'
-        env_set('CIBW_TEST_COMMAND', f'python {{project}}/scripts/gh_release.py{valgrind_text} test {{project}} {{package}}')
+        if inputs_PYMUPDF_SETUP_MUPDF_BUILD_TYPE not in ('-', None):
+            log(f'Setting PYMUPDF_SETUP_MUPDF_BUILD_TYPE to {inputs_PYMUPDF_SETUP_MUPDF_BUILD_TYPE!r}.')
+            env_set('PYMUPDF_SETUP_MUPDF_BUILD_TYPE', inputs_PYMUPDF_SETUP_MUPDF_BUILD_TYPE, pass_=True)
     
-    if pymupdf_dir != os.path.abspath( os.getcwd()):
-        log( f'Changing dir to {pymupdf_dir=}')
-        os.chdir( pymupdf_dir)
+        def set_cibuild_test():
+            log( f'set_cibuild_test(): {inputs_skeleton=}')
+            valgrind_text = ''
+            if valgrind:
+                valgrind_text = ' --valgrind 1'
+            env_set('CIBW_TEST_COMMAND', f'python {{project}}/scripts/gh_release.py{valgrind_text} test {{project}} {{package}}')
     
-    run('pip install cibuildwheel')
+        if pymupdf_dir != os.path.abspath( os.getcwd()):
+            log( f'Changing dir to {pymupdf_dir=}')
+            os.chdir( pymupdf_dir)
     
-    if inputs_flavours:
-        # Build and test PyMuPDF and PyMuPDFb wheels.
-        #
+        run('pip install cibuildwheel')
+    
+        # We include MuPDF build-time files.
+        flavour_d = True
         
-        # First build PyMuPDFb wheel. cibuildwheel will build a single wheel
-        # here, which will work with any python version on current OS.
-        #
-        env_set( 'PYMUPDF_SETUP_FLAVOUR', 'b', pass_=1)
-        run( f'cibuildwheel{platform_arg}', env_extra)
-        run( 'echo after flavour=b')
-        run( 'ls -l wheelhouse')
+        if PYMUPDF_SETUP_PY_LIMITED_API:
+            # Build one wheel with oldest python, then fake build with other python
+            # versions so we test everything.
+            log(f'{PYMUPDF_SETUP_PY_LIMITED_API=}')
+            env_pass('PYMUPDF_SETUP_PY_LIMITED_API')
+            # This isn't actually necessary - PYMUPDF_SETUP_PY_LIMITED_API is
+            # already in the environment - but we set it anyway for clarity.
+            env_set('PYMUPDF_SETUP_PY_LIMITED_API', PYMUPDF_SETUP_PY_LIMITED_API, pass_=1)
+            CIBW_BUILD_old = env_extra.get('CIBW_BUILD')
+            assert CIBW_BUILD_old is not None
+            env_set('CIBW_BUILD', 'cp38*')
+            log(f'Building single wheel.')
+            run( f'cibuildwheel{platform_arg}', env_extra=env_extra)
+            
+            # Fake-build with all python versions, using the wheel we have
+            # just created. This works by setting PYMUPDF_SETUP_URL_WHEEL
+            # which makes PyMuPDF's setup.py copy an existing wheel instead
+            # of building a wheel itself; it also copes with existing
+            # wheels having extra platform tags (from cibuildwheel's use of
+            # auditwheel).
+            #
+            env_set('PYMUPDF_SETUP_URL_WHEEL', f'file://wheelhouse/', pass_=True)
+            
+            set_cibuild_test()
+            env_set('CIBW_BUILD', CIBW_BUILD_old)
+            
+            # Disable cibuildwheels use of auditwheel. The wheel was repaired
+            # when it was created above so we don't need to do so again. This
+            # also avoids problems with musl wheels on a Linux glibc host where
+            # auditwheel fails with: `ValueError: Cannot repair wheel, because
+            # required library "libgcc_s-a3a07607.so.1" could not be located`.
+            #
+            env_set('CIBW_REPAIR_WHEEL_COMMAND', '')
+            
+            log(f'Testing on all python versions using wheels in wheelhouse/.')
+            run( f'cibuildwheel{platform_arg}', env_extra=env_extra)
+            
+        elif inputs_flavours:
+            # Build and test PyMuPDF and PyMuPDFb wheels.
+            #
+        
+            # First build PyMuPDFb wheel. cibuildwheel will build a single wheel
+            # here, which will work with any python version on current OS.
+            #
+            flavour = 'b'
+            if flavour_d:
+                # Include MuPDF build-time files.
+                flavour += 'd'
+            env_set( 'PYMUPDF_SETUP_FLAVOUR', flavour, pass_=1)
+            run( f'cibuildwheel{platform_arg}', env_extra)
+            run( 'echo after {flavour=}')
+            run( 'ls -l wheelhouse')
 
-        # Now set environment to build PyMuPDF wheels. cibuildwheel will build
-        # one for each Python version.
-        #
+            # Now set environment to build PyMuPDF wheels. cibuildwheel will build
+            # one for each Python version.
+            #
         
-        # Tell cibuildwheel not to use `auditwheel`, because it cannot cope
-        # with us deliberately putting required libraries into a different
-        # wheel.
-        #
-        # Also, `auditwheel addtag` says `No tags to be added` and terminates
-        # with non-zero. See: https://github.com/pypa/auditwheel/issues/439.
-        #
-        env_set('CIBW_REPAIR_WHEEL_COMMAND_LINUX', '')
-        env_set('CIBW_REPAIR_WHEEL_COMMAND_MACOS', '')
+            # Tell cibuildwheel not to use `auditwheel`, because it cannot cope
+            # with us deliberately putting required libraries into a different
+            # wheel.
+            #
+            # Also, `auditwheel addtag` says `No tags to be added` and terminates
+            # with non-zero. See: https://github.com/pypa/auditwheel/issues/439.
+            #
+            env_set('CIBW_REPAIR_WHEEL_COMMAND_LINUX', '')
+            env_set('CIBW_REPAIR_WHEEL_COMMAND_MACOS', '')
         
-        # We tell cibuildwheel to test these wheels, but also set
-        # CIBW_BEFORE_TEST to make it first run ourselves with the
-        # `pip_install` arg to install the PyMuPDFb wheel. Otherwise
-        # installation of PyMuPDF would fail because it lists the
-        # PyMuPDFb wheel as a prerequisite. We need to use `pip_install`
-        # because wildcards do not work on Windows, and we want to be
-        # careful to avoid incompatible wheels, e.g. 32 vs 64-bit wheels
-        # coexist during Windows builds.
-        #
-        env_set('CIBW_BEFORE_TEST', f'python scripts/gh_release.py pip_install wheelhouse/PyMuPDFb')
+            # We tell cibuildwheel to test these wheels, but also set
+            # CIBW_BEFORE_TEST to make it first run ourselves with the
+            # `pip_install` arg to install the PyMuPDFb wheel. Otherwise
+            # installation of PyMuPDF would fail because it lists the
+            # PyMuPDFb wheel as a prerequisite. We need to use `pip_install`
+            # because wildcards do not work on Windows, and we want to be
+            # careful to avoid incompatible wheels, e.g. 32 vs 64-bit wheels
+            # coexist during Windows builds.
+            #
+            env_set('CIBW_BEFORE_TEST', f'python scripts/gh_release.py pip_install wheelhouse/PyMuPDFb')
         
-        set_cibuild_test()
+            set_cibuild_test()
         
-        env_set( 'PYMUPDF_SETUP_FLAVOUR', 'p', pass_=1)
+            # Build main PyMuPDF wheel.
+            flavour = 'p'
+            env_set( 'PYMUPDF_SETUP_FLAVOUR', flavour, pass_=1)
+            run( f'cibuildwheel{platform_arg}', env_extra=env_extra)
         
-    else:
-        # Build and test wheels which contain everything.
-        #
-        set_cibuild_test()
-        env_set( 'PYMUPDF_SETUP_FLAVOUR', 'pb', pass_=1)
+        else:
+            # Build and test wheels which contain everything.
+            #
+            flavour = 'pb'
+            if flavour_d:
+                flavour += 'd'
+            set_cibuild_test()
+            env_set( 'PYMUPDF_SETUP_FLAVOUR', flavour, pass_=1)
     
-    run( f'cibuildwheel{platform_arg}', env_extra=env_extra)
+            run( f'cibuildwheel{platform_arg}', env_extra=env_extra)
     
-    run( 'ls -lt wheelhouse')
+        run( 'ls -lt wheelhouse')
 
 
-def build_pyodide_wheel( implementations, inputs_PYMUPDF_SETUP_MUPDF_BUILD):
+def build_pyodide_wheel(inputs_PYMUPDF_SETUP_MUPDF_BUILD):
     '''
     Build Pyodide wheel.
 
@@ -381,8 +512,6 @@ def build_pyodide_wheel( implementations, inputs_PYMUPDF_SETUP_MUPDF_BUILD):
     # Tell MuPDF to build for Pyodide.
     env_extra['OS'] = 'pyodide'
 
-    env_extra['PYMUPDF_SETUP_IMPLEMENTATIONS'] = implementations
-    
     # Build a single wheel without a separate PyMuPDFb wheel.
     env_extra['PYMUPDF_SETUP_FLAVOUR'] = 'pb'
     
@@ -416,7 +545,7 @@ def cpu_bits():
 #
 venv_name = f'venv-pymupdf-{platform.python_version()}-{cpu_bits()}'
 
-def venv( command=None, packages=None, quick=False):
+def venv( command=None, packages=None, quick=False, system_site_packages=False):
     '''
     Runs remaining args, or the specified command if present, in a venv.
     
@@ -430,16 +559,17 @@ def venv( command=None, packages=None, quick=False):
         install Python packages in it.
     '''
     command2 = ''
-    ssp = ''
     if platform.system() == 'OpenBSD':
         # libclang not available from pypi.org, but system py3-llvm package
         # works. `pip install` should be run with --no-build-isolation and
-        # explicit `pip install swig setuptools psutil`.
-        ssp = ' --system-site-packages'
+        # explicit `pip install swig psutil`.
+        system_site_packages = True
+        #ssp = ' --system-site-packages'
         log(f'OpenBSD: libclang not available from pypi.org.')
         log(f'OpenBSD: system package `py3-llvm` must be installed.')
         log(f'OpenBSD: creating venv with --system-site-packages.')
         log(f'OpenBSD: `pip install .../PyMuPDF` must be preceded by install of swig etc.')
+    ssp = ' --system-site-packages' if system_site_packages else ''
     if quick and os.path.isdir(venv_name):
         log(f'{quick=}: Not creating venv because directory already exists: {venv_name}')
         command2 += 'true'
@@ -520,8 +650,11 @@ def pyodide_setup(clean=False):
     # Clone emsdk.
     #
     dir_emsdk = 'emsdk'
-    if clean and os.path.exists(dir_emsdk):
-        shutil.rmtree( dir_emsdk, ignore_errors=1)
+    if clean:
+        shutil.rmtree(dir_emsdk, ignore_errors=1)
+        # 2024-06-25: old `.pyodide-xbuildenv` directory was breaking build, so
+        # important to remove it here.
+        shutil.rmtree('.pyodide-xbuildenv', ignore_errors=1)
     if not os.path.exists(dir_emsdk):
         command += f' && echo "### cloning emsdk.git"'
         command += f' && git clone https://github.com/emscripten-core/emsdk.git {dir_emsdk}'
@@ -632,9 +765,8 @@ def log(text, caller=0):
     filename    = frame_record.filename
     line        = frame_record.lineno
     function    = frame_record.function
-    prefix = f'{relpath(filename, pymupdf_dir)}:{function}(): '
-    print(textwrap.indent(text, prefix))
-    sys.stdout.flush()
+    prefix = f'{relpath(filename, pymupdf_dir)}:{line}:{function}(): '
+    print(textwrap.indent(text, prefix), flush=1)
 
 
 def run(command, env_extra=None, check=1, timeout=None):
@@ -654,10 +786,16 @@ def run(command, env_extra=None, check=1, timeout=None):
             leave processes running if timeout expires.
     '''
     env = None
+    message = 'Running: '
     if env_extra:
         env = os.environ.copy()
         env.update(env_extra)
-    log(f'Running: {command}')
+        message += '\n[environment:\n'
+        for n, v in env_extra.items():
+            message += f'    {n}={shlex.quote(v)}\n'
+        message += ']\n'
+    message += f'{command}'
+    log(message, caller=1)
     return subprocess.run(command, check=check, shell=1, env=env, timeout=timeout)
 
 
@@ -673,7 +811,7 @@ def platform_tag():
         assert 0, f'Unrecognised: {platform.system()=}'
 
 
-test_packages = 'pytest fontTools pymupdf-fonts flake8 pylint'
+test_packages = 'pytest fontTools pymupdf-fonts flake8 pylint codespell'
 if platform.system() == 'Windows' and cpu_bits() == 32:
     # No pillow wheel available, and doesn't build easily.
     pass
