@@ -17,7 +17,7 @@ Examples:
         Build and test using internal checkout of mupdf 1.24.x branch from Github.
 
 Usage:
-    scripts/test.py <options> <commands>
+    scripts/test.py <options> <command(s)>
 
 * Commands are handled in order, so for example `build` should usually be
   before `test`.
@@ -45,18 +45,21 @@ Options:
         PyMuPDF's `setup.py`.]
     -d
         Equivalent to `--build-type debug`.
+    -f 0|1
+        If 1 we also test alias `fitz` as well as `pymupdf`. Default is '0'.
     -i <implementations>
         Set PyMuPDF implementations to test.
         <implementations> must contain only these individual characters:
-             'c' - classic.
              'r' - rebased.
              'R' - rebased without optimisations.
-            Default is 'rR'. Also see `PyMuPDF:tests/run_compound.py`.
+            Default is 'r'. Also see `PyMuPDF:tests/run_compound.py`.
+    -k <expression>
+        Passed straight through to pytest's `-k`.
     -m <location> | --mupdf <location>
         Location of local mupdf/ directory or 'git:...' to be used
         when building PyMuPDF. [This sets environment variable
         PYMUPDF_SETUP_MUPDF_BUILD, which is used by PyMuPDF/setup.py. If not
-        specifed PyMuPDF will download its default mupdf .tgz.]
+        specified PyMuPDF will download its default mupdf .tgz.]
     -p <pytest-options>
         Set pytest options; default is ''.
     -t <names>
@@ -66,18 +69,27 @@ Options:
             -t tests/test_general.py::test_subset_fonts.
         To specify multiple tests, use comma-separated list and/or multiple `-t
         <names>` args.
-    -v
-        Avoid delay if venv directory already exists. We assume the existing
-        directory was created by us earlier and is a valid venv containing all
-        necessary packages.
+    -v 0|1|2
+        0 - do not use a venv.
+        1 - Use venv. If it already exists, we assume the existing directory
+            was created by us earlier and is a valid venv containing all
+            necessary packages; this saves a little time.
+        2 - use venv.
+        The default is 2.
     --build-isolation 0|1
         If true (the default on non-OpenBSD systems), we let pip create and use
         its own new venv to build PyMuPDF. Otherwise we force pip to use the
         current venv.
+    --build-flavour <build_flavour>
+        Combination of 'p', 'b', 'd'. See ../setup.py's description of
+        PYMUPDF_SETUP_FLAVOUR. Default is 'pb', i.e. self-contained PyMuPDF
+        wheels without MuPDF build-time files.
     --build-mupdf 0|1
         Whether to rebuild mupdf when we build PyMuPDF. Default is 1.
     --gdb 0|1
         Run tests under gdb.
+    --system-site-packages 0|1
+        If 1, use `--system-site-packages` when creating venv.
     --timeout <seconds>
         Sets timeout when running tests.
     --valgrind 0|1
@@ -92,6 +104,8 @@ Commands:
     test
         Runs PyMuPDF's pytest tests in venv. Default is to test rebased and
         unoptimised rebased; use `-i` to change this.
+    wheel
+        Build wheel.
 
 Environment:
     PYMUDF_SCRIPTS_TEST_options
@@ -100,6 +114,7 @@ Environment:
 
 import gh_release
 
+import glob
 import os
 import platform
 import re
@@ -114,6 +129,9 @@ pymupdf_dir = os.path.abspath( f'{__file__}/../..')
 
 def main(argv):
 
+    if github_workflow_unimportant():
+        return
+    
     if len(argv) == 1:
         show_help()
         return
@@ -121,14 +139,19 @@ def main(argv):
     build_isolation = None
     valgrind = False
     s = True
+    build_do = 'i'
     build_type = None
     build_mupdf = True
+    build_flavour = 'pb'
     gdb = False
-    implementations = None
+    test_fitz = False
+    implementations = 'r'
     test_names = list()
-    venv_quick = False
+    venv = 2
     pytest_options = None
     timeout = None
+    pytest_k = None
+    system_site_packages = False
     
     options = os.environ.get('PYMUDF_SCRIPTS_TEST_options', '')
     options = shlex.split(options)
@@ -149,6 +172,8 @@ def main(argv):
             build_isolation = int(next(args))
         elif arg == '-d':
             build_type = 'debug'
+        elif arg == '-f':
+            test_fitz = int(next(args))
         elif arg in ('-h', '--help'):
             show_help()
             return
@@ -160,14 +185,20 @@ def main(argv):
                 assert os.path.isdir(mupdf), f'Not a directory: {mupdf=}.'
                 mupdf = os.path.abspath(mupdf)
             os.environ['PYMUPDF_SETUP_MUPDF_BUILD'] = mupdf
+        elif arg == '-k':
+            pytest_k = next(args)
         elif arg == '-p':
             pytest_options  = next(args)
+        elif arg == '--system-site-packages':
+            system_site_packages = int(next(args))
         elif arg == '-t':
             test_names += next(args).split(',')
         elif arg == '--timeout':
             timeout = float(next(args))
         elif arg == '-v':
-            venv_quick = True
+            venv = int(next(args))
+        elif arg == '--build-flavour':
+            build_flavour = next(args)
         elif arg == '--build-mupdf':
             build_mupdf = int(next(args))
         elif arg == '--gdb':
@@ -183,7 +214,7 @@ def main(argv):
     
     commands = list()
     while 1:
-        assert arg in ('build', 'buildtest', 'test'), \
+        assert arg in ('build', 'buildtest', 'test', 'wheel'), \
                 f'Unrecognised command: {arg=}.'
         commands.append(arg)
         try:
@@ -191,20 +222,27 @@ def main(argv):
         except StopIteration:
             break
     
-    # We always want to run inside a venv.
-    if sys.prefix == sys.base_prefix:
+    venv_quick = (venv==1)
+    
+    # Run inside a venv.
+    if venv and sys.prefix == sys.base_prefix:
         # We are not running in a venv.
         log(f'Re-running in venv {gh_release.venv_name!r}.')
-        gh_release.venv( ['python'] + argv, quick=venv_quick)
+        gh_release.venv(
+                ['python'] + argv,
+                quick=venv_quick,
+                system_site_packages=system_site_packages,
+                )
         return
 
-    def do_build():
+    def do_build(wheel=False):
         build(
-                implementations=implementations,
                 build_type=build_type,
                 build_isolation=build_isolation,
                 venv_quick=venv_quick,
                 build_mupdf=build_mupdf,
+                build_flavour=build_flavour,
+                wheel=wheel,
                 )
     def do_test():
         test(
@@ -215,6 +253,8 @@ def main(argv):
                 pytest_options=pytest_options,
                 timeout=timeout,
                 gdb=gdb,
+                test_fitz=test_fitz,
+                pytest_k=pytest_k,
                 )
     
     for command in commands:
@@ -227,6 +267,8 @@ def main(argv):
         elif command == 'buildtest':
             do_build()
             do_test()
+        elif command == 'wheel':
+            do_build(wheel=True)
         else:
             assert 0
 
@@ -247,6 +289,21 @@ def show_help():
     print(venv_info())
 
 
+def github_workflow_unimportant():
+    '''
+    Returns true if we are running a Github scheduled workflow but in a
+    repository not called 'PyMuPDF'. This can be used to avoid consuming
+    unnecessary Github minutes running workflows on non-main repositories such
+    as ArtifexSoftware/PyMuPDF-julian.
+    '''
+    GITHUB_EVENT_NAME = os.environ.get('GITHUB_EVENT_NAME')
+    GITHUB_REPOSITORY = os.environ.get('GITHUB_REPOSITORY')
+    if GITHUB_EVENT_NAME == 'schedule' and GITHUB_REPOSITORY != 'pymupdf/PyMuPDF':
+            log(f'## This is an unimportant Github workflow: a scheduled event, not in the main repository `pymupdf/PyMuPDF`.')
+            log(f'## {GITHUB_EVENT_NAME=}.')
+            log(f'## {GITHUB_REPOSITORY=}.')
+            return True
+
 def venv_info(pytest_args=None):
     '''
     Returns string containing information about the venv we use and how to
@@ -280,11 +337,12 @@ def venv_info(pytest_args=None):
 
 
 def build(
-        implementations=None,
         build_type=None,
         build_isolation=None,
         venv_quick=False,
         build_mupdf=True,
+        build_flavour='pb',
+        wheel=False,
         ):
     '''
     Args:
@@ -317,26 +375,25 @@ def build(
         import setup
         names = setup.get_requires_for_build_wheel()
         del sys.path[0]
-        names = ' '.join(names)
-        if venv_quick:
-            log(f'{venv_quick=}: Not installing packages with pip: {names}')
-        else:
-            gh_release.run( f'python -m pip install --upgrade {names}')
+        if names:
+            names = ' '.join(names)
+            if venv_quick:
+                log(f'{venv_quick=}: Not installing packages with pip: {names}')
+            else:
+                gh_release.run( f'python -m pip install --upgrade {names}')
         build_isolation_text = ' --no-build-isolation'
     
     env_extra = dict()
-    if implementations:
-        v = ''
-        if 'c' in implementations:
-            v += 'a'
-        if 'r' in implementations or 'R' in implementations:
-            v += 'b'
-        env_extra['PYMUPDF_SETUP_IMPLEMENTATIONS'] = v
     if not build_mupdf:
         env_extra['PYMUPDF_SETUP_MUPDF_REBUILD'] = '0'
     if build_type:
         env_extra['PYMUPDF_SETUP_MUPDF_BUILD_TYPE'] = build_type
-    gh_release.run(f'pip install{build_isolation_text} -vv {pymupdf_dir}', env_extra=env_extra)
+    if build_flavour:
+        env_extra['PYMUPDF_SETUP_FLAVOUR'] = build_flavour
+    if wheel:
+        gh_release.run(f'pip wheel{build_isolation_text} -v {pymupdf_dir}', env_extra=env_extra)
+    else:
+        gh_release.run(f'pip install{build_isolation_text} -v {pymupdf_dir}', env_extra=env_extra)
 
 
 def test(
@@ -347,6 +404,8 @@ def test(
         pytest_options=None,
         timeout=None,
         gdb=False,
+        test_fitz=True,
+        pytest_k=None
         ):
     '''
     Args:
@@ -355,13 +414,15 @@ def test(
         valgrind:
             See top-level option `--valgrind`.
         venv_quick:
-            See top-level option `-v`.
+            .
         test_names:
             See top-level option `-t`.
         pytest_options:
             See top-level option `-p`.
         gdb:
             See top-level option `--gdb`.
+        test_fitz:
+            See top-level option `-f`.
     '''
     pymupdf_dir_rel = gh_release.relpath(pymupdf_dir)
     if pytest_options is None:
@@ -369,6 +430,8 @@ def test(
             pytest_options = '-s -vv'
         else:
             pytest_options = ''
+    if pytest_k:
+        pytest_options += f' -k {shlex.quote(pytest_k)}'
     pytest_arg = ''
     if test_names:
         for test_name in test_names:
@@ -381,7 +444,7 @@ def test(
         if venv_quick:
             log(f'{venv_quick=}: Not installing test packages: {gh_release.test_packages}')
         else:
-            gh_release.run(f'pip install {gh_release.test_packages}')
+            gh_release.run(f'pip install --upgrade {gh_release.test_packages}')
         run_compound_args = ''
         if implementations:
             run_compound_args += f' -i {implementations}'
@@ -391,7 +454,7 @@ def test(
         if valgrind:
             log('Installing valgrind.')
             gh_release.run(f'sudo apt update')
-            gh_release.run(f'sudo apt install valgrind')
+            gh_release.run(f'sudo apt install --upgrade valgrind')
             gh_release.run(f'valgrind --version')
         
             log('Running PyMuPDF tests under valgrind.')
@@ -407,13 +470,38 @@ def test(
         elif gdb:
             command = f'{python} {pymupdf_dir_rel}/tests/run_compound.py{run_compound_args} gdb --args {python} -m pytest {pytest_options} {pytest_arg}'
         elif platform.system() == 'Windows':
-            # `python -m pytest` doesn' seem to work.
+            # `python -m pytest` doesn't seem to work.
            command = f'{python} {pymupdf_dir_rel}/tests/run_compound.py{run_compound_args} pytest {pytest_options} {pytest_arg}'
         else:
             # On OpenBSD `pip install pytest` doesn't seem to install the pytest
             # command, so we use `python -m pytest ...`.
             command = f'{python} {pymupdf_dir_rel}/tests/run_compound.py{run_compound_args} {python} -m pytest {pytest_options} {pytest_arg}'
         
+        # Always start by removing any test_*_fitz.py files.
+        for p in glob.glob(f'{pymupdf_dir_rel}/tests/test_*_fitz.py'):
+            print(f'Removing {p=}')
+            os.remove(p)
+        if test_fitz:
+            # Create copies of each test file, modified to use `pymupdf`
+            # instead of `fitz`.
+            for p in glob.glob(f'{pymupdf_dir_rel}/tests/test_*.py'):
+                if os.path.basename(p).startswith('test_fitz_'):
+                    # Don't recursively generate test_fitz_fitz_foo.py,
+                    # test_fitz_fitz_fitz_foo.py, ... etc.
+                    continue
+                branch, leaf = os.path.split(p)
+                p2 = f'{branch}/{leaf[:5]}fitz_{leaf[5:]}'
+                print(f'Converting {p=} to {p2=}.')
+                with open(p, encoding='utf8') as f:
+                    text = f.read()
+                text2 = re.sub("([^\'])\\bpymupdf\\b", '\\1fitz', text)
+                if p.replace(os.sep, '/') == f'{pymupdf_dir_rel}/tests/test_docs_samples.py'.replace(os.sep, '/'):
+                    assert text2 == text
+                else:
+                    assert text2 != text, f'Unexpectedly unchanged when creating {p!r} => {p2!r}'
+                with open(p2, 'w', encoding='utf8') as f:
+                    f.write(text2)
+        
         log(f'Running tests with tests/run_compound.py and pytest.')
         gh_release.run(command, env_extra=env_extra, timeout=timeout)
             
