@@ -5,7 +5,7 @@ Fill a given text in a rectangle on some PDF page using
 
 Check text is indeed contained in given rectangle.
 """
-import fitz
+import pymupdf
 
 text = """Der Kleine Schwertwal (Pseudorca crassidens), auch bekannt als Unechter oder Schwarzer Schwertwal, ist eine Art der Delfine (Delphinidae) und der einzige rezente Vertreter der Gattung Pseudorca.
 
@@ -18,18 +18,18 @@ Sie sind in allen Ozeanen gemäßigter, subtropischer und tropischer Breiten beh
 
 def test_textbox1():
     """Use TextWriter for text insertion."""
-    doc = fitz.open()
+    doc = pymupdf.open()
     page = doc.new_page()
-    rect = fitz.Rect(50, 50, 400, 400)
+    rect = pymupdf.Rect(50, 50, 400, 400)
     blue = (0, 0, 1)
-    tw = fitz.TextWriter(page.rect, color=blue)
+    tw = pymupdf.TextWriter(page.rect, color=blue)
     tw.fill_textbox(
         rect,
         text,
-        align=fitz.TEXT_ALIGN_LEFT,
+        align=pymupdf.TEXT_ALIGN_LEFT,
         fontsize=12,
     )
-    tw.write_text(page, morph=(rect.tl, fitz.Matrix(1, 1)))
+    tw.write_text(page, morph=(rect.tl, pymupdf.Matrix(1, 1)))
     # check text containment
     assert page.get_text() == page.get_text(clip=rect)
     page.write_text(writers=tw)
@@ -37,16 +37,16 @@ def test_textbox1():
 
 def test_textbox2():
     """Use basic text insertion."""
-    doc = fitz.open()
+    doc = pymupdf.open()
     ocg = doc.add_ocg("ocg1")
     page = doc.new_page()
-    rect = fitz.Rect(50, 50, 400, 400)
-    blue = fitz.utils.getColor("lightblue")
-    red = fitz.utils.getColorHSV("red")
+    rect = pymupdf.Rect(50, 50, 400, 400)
+    blue = pymupdf.utils.getColor("lightblue")
+    red = pymupdf.utils.getColorHSV("red")
     page.insert_textbox(
         rect,
         text,
-        align=fitz.TEXT_ALIGN_LEFT,
+        align=pymupdf.TEXT_ALIGN_LEFT,
         fontsize=12,
         color=blue,
         oc=ocg,
@@ -57,21 +57,21 @@ def test_textbox2():
 
 def test_textbox3():
     """Use TextWriter for text insertion."""
-    doc = fitz.open()
+    doc = pymupdf.open()
     page = doc.new_page()
-    font = fitz.Font("cjk")
-    rect = fitz.Rect(50, 50, 400, 400)
+    font = pymupdf.Font("cjk")
+    rect = pymupdf.Rect(50, 50, 400, 400)
     blue = (0, 0, 1)
-    tw = fitz.TextWriter(page.rect, color=blue)
+    tw = pymupdf.TextWriter(page.rect, color=blue)
     tw.fill_textbox(
         rect,
         text,
-        align=fitz.TEXT_ALIGN_LEFT,
+        align=pymupdf.TEXT_ALIGN_LEFT,
         font=font,
         fontsize=12,
         right_to_left=True,
     )
-    tw.write_text(page, morph=(rect.tl, fitz.Matrix(1, 1)))
+    tw.write_text(page, morph=(rect.tl, pymupdf.Matrix(1, 1)))
     # check text containment
     assert page.get_text() == page.get_text(clip=rect)
     doc.scrub()
@@ -80,59 +80,64 @@ def test_textbox3():
 
 def test_textbox4():
     """Use TextWriter for text insertion."""
-    doc = fitz.open()
+    doc = pymupdf.open()
     ocg = doc.add_ocg("ocg1")
     page = doc.new_page()
-    rect = fitz.Rect(50, 50, 400, 600)
+    rect = pymupdf.Rect(50, 50, 400, 600)
     blue = (0, 0, 1)
-    tw = fitz.TextWriter(page.rect, color=blue)
+    tw = pymupdf.TextWriter(page.rect, color=blue)
     tw.fill_textbox(
         rect,
         text,
-        align=fitz.TEXT_ALIGN_LEFT,
+        align=pymupdf.TEXT_ALIGN_LEFT,
         fontsize=12,
-        font=fitz.Font("cour"),
+        font=pymupdf.Font("cour"),
         right_to_left=True,
     )
-    tw.write_text(page, oc=ocg, morph=(rect.tl, fitz.Matrix(1, 1)))
+    tw.write_text(page, oc=ocg, morph=(rect.tl, pymupdf.Matrix(1, 1)))
     # check text containment
     assert page.get_text() == page.get_text(clip=rect)
 
 
 def test_textbox5():
     """Using basic text insertion."""
-    fitz.TOOLS.set_small_glyph_heights(True)
-    doc = fitz.open()
-    page = doc.new_page()
-    r = fitz.Rect(100, 100, 150, 150)
-    text = "words and words and words and more words..."
-    rc = -1
-    fontsize = 12
-    page.draw_rect(r)
-    while rc < 0:
-        rc = page.insert_textbox(
-            r,
-            text,
-            fontsize=fontsize,
-            align=fitz.TEXT_ALIGN_JUSTIFY,
-        )
-        fontsize -= 0.5
+    small_glyph_heights0 = pymupdf.TOOLS.set_small_glyph_heights()
+    pymupdf.TOOLS.set_small_glyph_heights(True)
+    try:
+        doc = pymupdf.open()
+        page = doc.new_page()
+        r = pymupdf.Rect(100, 100, 150, 150)
+        text = "words and words and words and more words..."
+        rc = -1
+        fontsize = 12
+        page.draw_rect(r)
+        while rc < 0:
+            rc = page.insert_textbox(
+                r,
+                text,
+                fontsize=fontsize,
+                align=pymupdf.TEXT_ALIGN_JUSTIFY,
+            )
+            fontsize -= 0.5
 
-    blocks = page.get_text("blocks")
-    bbox = fitz.Rect(blocks[0][:4])
-    assert bbox in r
+        blocks = page.get_text("blocks")
+        bbox = pymupdf.Rect(blocks[0][:4])
+        assert bbox in r
+    finally:
+        # Must restore small_glyph_heights, otherwise other tests can fail.
+        pymupdf.TOOLS.set_small_glyph_heights(small_glyph_heights0)
 
 
 def test_2637():
     """Ensure correct calculation of fitting text."""
-    doc = fitz.open()
+    doc = pymupdf.open()
     page = doc.new_page()
     text = (
         "The morning sun painted the sky with hues of orange and pink. "
         "Birds chirped harmoniously, greeting the new day. "
         "Nature awakened, filling the air with life and promise."
     )
-    rect = fitz.Rect(50, 50, 500, 280)
+    rect = pymupdf.Rect(50, 50, 500, 280)
     fontsize = 50
     rc = -1
     while rc < 0:  # look for largest font size that makes the text fit
@@ -141,7 +146,7 @@ def test_2637():
 
     # confirm text won't lap outside rect
     blocks = page.get_text("blocks")
-    bbox = fitz.Rect(blocks[0][:4])
+    bbox = pymupdf.Rect(blocks[0][:4])
     assert bbox in rect
 
 
@@ -157,17 +162,17 @@ def test_htmlbox1():
     We try to insert into a rectangle that is too small, setting
     scale=False and confirming we have a negative return code.
     """
-    if not hasattr(fitz, "mupdf"):
+    if not hasattr(pymupdf, "mupdf"):
         print("'test_htmlbox1' not executed in classic.")
         return
 
-    rect = fitz.Rect(100, 100, 200, 200)  # this only works with scale=True
+    rect = pymupdf.Rect(100, 100, 200, 200)  # this only works with scale=True
 
     base_text = """Lorem ipsum dolor sit amet, consectetur adipisici elit, sed eiusmod tempor incidunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquid ex ea commodi consequat. Quis aute iure reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint obcaecat cupiditat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."""
 
     text = """Lorem ipsum dolor sit amet, consectetur adipisici elit, sed eiusmod tempor incidunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation <b>ullamco</b> <i>laboris</i> nisi ut aliquid ex ea commodi consequat. Quis aute iure reprehenderit in <span style="color: #0f0;font-weight:bold;">voluptate</span> velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint obcaecat cupiditat non proident, sunt in culpa qui <a href="https://www.artifex.com">officia</a> deserunt mollit anim id est laborum."""
 
-    doc = fitz.Document()
+    doc = pymupdf.Document()
 
     for rot in (0, 90, 180, 270):
         wdirs = ((1, 0), (0, -1), (-1, 0), (0, 1))  # all writing directions
@@ -194,7 +199,7 @@ def test_htmlbox1():
                 assert wdir == wdirs[page.number]
                 for s in l["spans"]:
                     stext = s["text"]
-                    color = fitz.sRGB_to_pdf(s["color"])
+                    color = pymupdf.sRGB_to_pdf(s["color"])
                     bold = bool(s["flags"] & 16)
                     italic = bool(s["flags"] & 2)
                     if stext in ("ullamco", "laboris", "voluptate"):
@@ -202,15 +207,15 @@ def test_htmlbox1():
                         if stext == "ullamco":
                             assert bold is True
                             assert italic is False
-                            assert color == fitz.pdfcolor["black"]
+                            assert color == pymupdf.pdfcolor["black"]
                         elif stext == "laboris":
                             assert bold is False
                             assert italic is True
-                            assert color == fitz.pdfcolor["black"]
+                            assert color == pymupdf.pdfcolor["black"]
                         elif stext == "voluptate":
                             assert bold is True
                             assert italic is False
-                            assert color == fitz.pdfcolor["green"]
+                            assert color == pymupdf.pdfcolor["green"]
                     else:
                         assert bold is False
                         assert italic is False
@@ -220,12 +225,12 @@ def test_htmlbox1():
 
 def test_htmlbox2():
     """Test insertion without scaling"""
-    if not hasattr(fitz, "mupdf"):
+    if not hasattr(pymupdf, "mupdf"):
         print("'test_htmlbox2' not executed in classic.")
         return
 
-    doc = fitz.open()
-    rect = fitz.Rect(100, 100, 200, 200)  # large enough to hold text
+    doc = pymupdf.open()
+    rect = pymupdf.Rect(100, 100, 200, 200)  # large enough to hold text
     page = doc.new_page()
     bottoms = set()
     for rot in (0, 90, 180, 270):
@@ -240,13 +245,13 @@ def test_htmlbox2():
 
 def test_htmlbox3():
     """Test insertion with opacity"""
-    if not hasattr(fitz, "mupdf"):
+    if not hasattr(pymupdf, "mupdf"):
         print("'test_htmlbox3' not executed in classic.")
         return
 
-    rect = fitz.Rect(100, 250, 300, 350)
+    rect = pymupdf.Rect(100, 250, 300, 350)
     text = """<span style="color:red;font-size:20px;">Just some text.</span>"""
-    doc = fitz.open()
+    doc = pymupdf.open()
     page = doc.new_page()
 
     # insert some text with opacity
@@ -255,3 +260,14 @@ def test_htmlbox3():
     # lowlevel-extract inserted text to access opacity
     span = page.get_texttrace()[0]
     assert span["opacity"] == 0.5
+
+
+def test_3559():
+    if pymupdf.mupdf_version_tuple < (1, 24, 4):
+        print(f'test_3559(): Not running because mupdf known to SEGV.')
+        return
+    doc = pymupdf.Document()
+    page = doc.new_page()
+    text_insert="""<body><h3></h3></body>"""
+    rect = pymupdf.Rect(100, 100, 200, 200)
+    page.insert_htmlbox(rect, text_insert)
